%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Do not modify this file since it was automatically generated from:
% 
%  
% 
% on Tue May 03 13:27:14 2005.
% 
% Generator was the Rdoc class, which is part of the R.oo package written
% by Henrik Bengtsson, 2001-2004.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 \name{Object}
\docType{class}
\alias{Object}
\keyword{classes}


 \title{The root class that every class must inherit from}

 \description{
  R.oo\cr
  \bold{Class Object}\cr

  public class \bold{Object}\cr

  \code{Object} is the root class of all other classes. All classes
  \emph{must} extends this class, directly or indirectly, which means
  that they all will inherit the methods in this class.
 }

 \usage{Object(core=NA)}

 \arguments{
   \item{core}{The core value of each \emph{reference} referering to the
     Object. By default, this is just the smallest possible \R object, but
     there are situations where it is useful to have another kind of core,
     which is the case with the Class class.
     \emph{Note that this value is belonging to the reference variable
     and not to the Object, which means it can not be referenced.}}
 }

 \section{Fields and Methods}{
  \bold{Methods:}\cr
\tabular{rll}{
 \tab \code{\link[R.oo:DOLLAR.Object]{$}} \tab Makes the fields and methods of an Object accessable via the \$ and the [[ operator.\cr
 \tab \code{\link[R.oo:DOLLARLT_-.Object]{$<-}} \tab Makes the fields and methods of an Object assignable via the \$<- and the [[<- operator.\cr
 \tab \code{[[} \tab  -\cr
 \tab \code{[[<-} \tab  -\cr
 \tab \code{\link[R.oo:as.character.Object]{as.character}} \tab Gets a character string representing the object.\cr
 \tab \code{\link[R.oo:attach.Object]{attach}} \tab Attach an Object to the \R search path.\cr
 \tab \code{\link[R.oo:clone.Object]{clone}} \tab Clones an Object.\cr
 \tab \code{\link[R.oo:detach.Object]{detach}} \tab Detach an Object from the \R search path.\cr
 \tab \code{\link[R.oo:equals.Object]{equals}} \tab Compares an object with another.\cr
 \tab \code{\link[R.oo:extend.Object]{extend}} \tab Extends another class.\cr
 \tab \code{\link[R.oo:finalize.Object]{finalize}} \tab Finalizer methods called when object is clean out.\cr
 \tab \code{\link[R.oo:getFields.Object]{getFields}} \tab Returns the field names of an Object.\cr
 \tab \code{\link[R.oo:getInstanciationTime.Object]{getInstanciationTime}} \tab Gets the time when the object was instanciated.\cr
 \tab \code{\link[R.oo:getStaticInstance.Object]{getStaticInstance}} \tab Gets the static instance of this objects class.\cr
 \tab \code{\link[R.oo:hasField.Object]{hasField}} \tab Checks if a field exists or not.\cr
 \tab \code{\link[R.oo:hashCode.Object]{hashCode}} \tab Gets a hash code for the Object.\cr
 \tab \code{\link[R.oo:ll.Object]{ll}} \tab Generates a list of informative properties of all members of an Object.\cr
 \tab \code{\link[R.oo:load.Object]{load}} \tab Static method to load an Object from a file or a connection.\cr
 \tab \code{\link[R.oo:objectSize.Object]{objectSize}} \tab Gets the size of the Object in bytes.\cr
 \tab \code{\link[R.oo:print.Object]{print}} \tab Prints an Object.\cr
 \tab \code{\link[R.oo:save.Object]{save}} \tab Saves an Object to a file or a connection.\cr
}



 }

 \examples{
#########################################################################
# Defines the class Person with private fields .name and .age, and
# with methods print(), getName(), setName(), getAge() and setAge().
#########################################################################
setConstructorS3("Person", function(name, age) {
  if (missing(name)) name <- NA;
  if (missing(age))  age <- NA;

  extend(Object(), "Person",
    .name=name,
    .age=age
  )
})


setMethodS3("as.character", "Person", function(this) {
  paste(this$.name, "is", as.integer(this$.age), "years old.");
})

setMethodS3("equals", "Person", function(this, obj) {
  ( identical(data.class(this), data.class(obj)) &&
    identical(this$getName(), obj$getName()) &&
    identical(this$getAge() , obj$getAge() )    );
})

setMethodS3("hashCode", "Person", function(this) {
  # Get the hashCode() of the '.name' and the '.age' fields
  # using hashCode.default().
  hashCode(this$.name) * hashCode(this$.age);
})

setMethodS3("getName", "Person", function(this) {
  this$.name;
})

setMethodS3("setName", "Person", function(this, newName) {
  throw("It is not possible to change the name of a Person.");
})

setMethodS3("getAge", "Person", function(this) {
  this$.age;
})

setMethodS3("setAge", "Person", function(this, newAge) {
  if (!is.numeric(newAge))
    throw("Age must be numeric: ", newAge);
  if (newAge < 0)
    throw("Trying to set a negative age: ", newAge);
  this$.age <- newAge;
})




#########################################################################
# Code demonstrating different properties of the Object class using
# the example class Person.
#########################################################################

# Create an object (instance of) the class Person.
p1 <- Person("Dalai Lama", 67)

# 'p1' is an Object of class Person.
print(data.class(p1))  # "Person"

# Prints information about the Person object.
print(p1)            # "Dalai Lama is 67 years old."

# or equivalent (except that no generic method has to exist):

p1$print()           # "Dalai Lama is 67 years old."

# Shows that no generic method is required if the \$ operator is used:
print(p1$getName())  # "Dalai Lama"

# The following will call p1$getName() since there exists a get-()
# method for the 'name' property.
print(p1$name)       # "Dalai Lama"

# and equivalent when using the [[ operator.
print(p1[["name"]])  # "Dalai Lama"

# The following shows that p1$setName(68) is called, simply because
# there exists a set-() method for the 'name' property.
p1$age <- 68         # Will call p1$setAge(68)

# Shows that the age of the Person has been updated:
print(p1)            # "Dalai Lama is 68 years old."

# If there would not exists such a set-() method or field a new
# field would be created:
p1$country <- "Tibet"

# Lists all (non-private) members of the Person object:
print(ll(p1))

# which gives
#      member class      mode    typeof length  dim bytes
#   1 country  NULL character character      1 NULL    44

# The following will call p1$setName("Lalai Dama") which will
# throw an exception saying one can not change the name of
# a Person.
tryCatch(p1$name <- "Lalai Dama", error=print)

# The following will call p1$setAge(-4) which will throw an
# exception saying that the age must be a non-negative number.
tryCatch(p1$age <- -100, error=print)

# Attaches Object 'p1' to the search path.
attach(p1)

# Accesses the newly created field 'country'.
print(country)       # "Tibet"

# Detaches Object 'p1' from the search path. Note that all
# modifications to 'country' are lost.
country <- "Sweden"
detach(p1)
print(p1$country)    # "Tibet"


# Saves the Person object to a tempory file.
filename <- tempfile("R.methodsS3.example")
save(p1, filename)

# Deletes the object
rm(p1)

# Loads an Object (of "unknown" class) from file using the
# static method load() of class Object.
obj <- Object$load(filename)

# Prints information about the new Object.
print(obj)

# Lists all (non-private) members of the new Object.
print(ll(obj))
}

 \author{Henrik Bengtsson \url{http://www.braju.com/R/}}

 \references{
   [1] Henrik Bengtsson, \emph{The R.oo package - Object-Oriented Programming
       with References Using Standard R Code}, In Kurt Hornik, Friedrich
       Leisch and Achim Zeileis, editors, Proceedings of the 3rd
       International Workshop on Distributed Statistical Computing
       (DSC 2003), March 20-22, Vienna, Austria.
       \url{http://www.ci.tuwien.ac.at/Conferences/DSC-2003/Proceedings/}\cr
 }

 \keyword{programming}
 \keyword{methods}
